package binomialheap;

/**
 * A BinomialHeap is a dispenser whose accessible data item is the highest
 * priority data item..
 * New data items can be added with the add() method.
 * The highest priority data item is returned by the get() method and is removed
 * and returned by the remove() method.
 */
public class BinomialHeap
        implements Dispenser {

  private BinomialNode treeListTrailer;
  private BinomialNode highestPriorityTrailer;

  /**
   * new BinomialHeap() returns an empty binomial heap.
   */
  public BinomialHeap() {
  }

  /** bh.isEmpty() returns true if disp contains no data items.
   * @return true if there are no data items
   */
  public boolean isEmpty() {
    return treeListTrailer == null;
  }

  /**
   * bh.get() returns the highest priority item of bh.
   * @return the highest priority data item
   */
  public Data get() {
    assert !isEmpty() :
            "Precondition violation: dispenser is empty";
    return highestPriorityTrailer.getSibling().getData();
  }

  /**
   * bh.add(item) adds item to bh.
   * @param item the data item to be added
   */
  public void add(Data item) {
    int oldCount = BinomialNode.getListNodeCount(treeListTrailer);
    // FIXME - add code.
    // Do not modify the following code.
    int newCount = BinomialNode.getListNodeCount(treeListTrailer);
    assert newCount == oldCount + 1 :
            "Add check: new count is not old count + 1";
  }

  /**
   * bh.remove() removes and returns the highest priority data item of bh.
   * @return the highest priority data item
   */
  public Data remove() {
    assert !isEmpty() :
            "Precondition violation: dispenser is empty";
    int oldCount = BinomialNode.getListNodeCount(treeListTrailer);
    // FIXME - modify next line of code and add more code.  Take care.
    Data highestPriorityData = null;
    // Do not modify the following code.
    int newCount = BinomialNode.getListNodeCount(treeListTrailer);
    assert newCount == oldCount - 1 :
            "Remove check: new count is not old count - 1";
    return highestPriorityData;
  }

}
