package binomialheap;

/**
 * A Dispenser is a container for data items.
 * New data items can be added with the add() method.
 * A dispenser selects one of its data items as the accessible item.
 * Each implementation determines its own policy for selecting the accessible
 * data items.
 * This data item is returned by the get() method and is removed and returned by
 * the remove() method.
 */
public interface Dispenser {

  /** disp.isEmpty() returns true if disp contains no data items.
   * @return true if there are no data items
   */
  public boolean isEmpty();

  /**
   * disp.get() returns the accessible item of disp.
   * @return the accessible data item
   */
  public Data get();

  /**
   * disp.add(item) adds item to disp.
   * @param item the data item to be added
   */
  public void add(Data item);

  /**
   * disp.remove() removes and returns the accessible data item of disp.
   * @return the accessible data item
   */
  public Data remove();

}
