package binomialheap;

import java.util.HashMap;
import java.util.Map;
import java.util.Random;

/**
 * This is a test program for the BinomialHeap class.
 */
public class Main {

  private BinomialHeap heap;
  private Map<String, Data> dataItems;
  private int currentId;
  private Random rng;

  /**
   * createData() returns a new data item with random priority.
   * Its identifier is generated automatically.
   * @return a new data item
   */
  private Data createData() {
    String id = "#" + currentId;
    currentId++;
    int priority = rng.nextInt(100);
    return new Data(id, priority);
  }

  /**
   * addItem() adds a new data item to the test binomial heap and sends reports
   * before and after to System.out.
   */
  public void addItem() {
    Data item = createData();
    dataItems.put(item.getIdentifier(), item);
    System.out.println("Adding " + item);
    heap.add(item);
    System.out.println("  Add done");
  }

  /**
   * removeItem() removes a data item from the test binomial heap and sends
   * reports before and after to System.out.
   */
  public void removeItem() {
    System.out.println("Removing an item");
    Data item = heap.remove();
    Data savedItem = dataItems.remove(item.getIdentifier());
    assert savedItem != null :
            "Test failure: Removed item must have been removed earlier";
    System.out.println("  The removed item is " + item);
  }

  /**
   * tester.test(count) adds count data items to the test binomial heap and then
   * removes them all.
   * Reports are sent to System.out before and after each add or remove.
   * @param count
   */
  public void test(int count) {
    for (int i = 0; i < count; i++) {
      addItem();
    }
    for (int i = 0; i < count; i++) {
      removeItem();
    }
    assert heap.isEmpty() :
            "Test failure: the heap is not empty";
  }

  public Main() {
    heap = new BinomialHeap();
    dataItems = new HashMap<String, Data>();
    currentId = 0;
    rng = new Random();
  }

  /**
   * @param args the command line arguments
   */
  public static void main(String[] args) {
    Main tester = new Main();
    tester.test(10);
  }

}
